# -*- coding:utf-8 -*-
"""
# Date: 2020-10-6
@ Author: Jean

@inproceedings{he-choi-2021-stem,
    title = "The Stem Cell Hypothesis: Dilemma behind Multi-Task Learning with Transformer Encoders",
    author = "He, Han and Choi, Jinho D.",
    booktitle = "Proceedings of the 2021 Conference on Empirical Methods in Natural Language Processing",
    month = nov,
    year = "2021",
    address = "Online and Punta Cana, Dominican Republic",
    publisher = "Association for Computational Linguistics",
    url = "https://aclanthology.org/2021.emnlp-main.451",
    pages = "5555--5577",
    abstract = "Multi-task learning with transformer encoders (MTL) has emerged as a powerful technique to improve performance on closely-related tasks for both accuracy and efficiency while a question still remains whether or not it would perform as well on tasks that are distinct in nature. We first present MTL results on five NLP tasks, POS, NER, DEP, CON, and SRL, and depict its deficiency over single-task learning. We then conduct an extensive pruning analysis to show that a certain set of attention heads get claimed by most tasks during MTL, who interfere with one another to fine-tune those heads for their own objectives. Based on this finding, we propose the Stem Cell Hypothesis to reveal the existence of attention heads naturally talented for many tasks that cannot be jointly trained to create adequate embeddings for all of those tasks. Finally, we design novel parameter-free probes to justify our hypothesis and demonstrate how attention heads are transformed across the five tasks during MTL through label analysis.",
}

"""

import hanlp
# from hanlp.common.trie import Trie  # HanLP2.0 -->2.1
from hanlp_trie.trie import Trie
import json,re
import time, random, math
import pandas as pd
from igraph import *
import collections


# 装入用户自定义词典
types = {"word":str,"pos":str}
dicZdy = pd.read_csv('C:/Users/Jean/Documents/Python Scripts/词汇导入demo-HanLP.csv', sep=',',\
                     header=None, dtype=types, encoding="GBK")
# dicZdy.columns = ["word","pos"]
dicZdy = dict(collections.OrderedDict(zip(dicZdy.iloc[:,0],dicZdy.iloc[:,1])))
# 挂载自定义词典
trie = Trie()
# key是自定义词， value可以是词性等，由业务定义。
#trie.update({'单唾液酸四己糖神经节苷脂钠': 'NN'})
trie.update(dicZdy)

# types = {"Tag":str,"Relation":str, "Description":str, "Example":str}
# synRelations = pd.read_csv('D:/temp/增值税自开票2017/句法依存关系.csv', sep=',',dtype=types, encoding="UTF-8")


# 自定义函数，先对文本用自定义词典分割一次，提取自定义词
def split_sents(text: str, trie: Trie):
    #words = trie.parse_longest(text)     # HandLP2.0 -->2.1
    words = trie.parse_longest2(text)    
    sents = []
    pre_start = 0
    offsets = []
    for word, value, start, end in words:
        if pre_start != start:
            sents.append(text[pre_start: start])
            offsets.append(pre_start)
        pre_start = end
    if pre_start != len(text):
        sents.append(text[pre_start:])
        offsets.append(pre_start)
    return sents, offsets, words


# 自定义函数，再合并自定义分词及词性标注，剩余部分的词性分词、词性标注
def merge_parts(tokens, postags, offsets, words):
    tps = []
    for i in range(len(tokens)):
        tps2 = [(token, postag) for (token, postag) in zip(tokens[i], postags[i])]
        tps.append(tps2)
    #items = [(i, p) for (i, p) in zip(offsets, tokens)]
    items = [(i, ts) for (i, ts) in zip(offsets, tps)]                
    # items += [(start, [word]) for (word, value, start, end) in words]
    # In case you need the tag, use the following line instead
    items += [(start, [(word, value)]) for (word, value, start, end) in words]
    return [each for x in sorted(items) for each in x[1]]
       

# 装入HanLP使用的分词、词性标注、依存句法分析等各种模型, 96秒，有点慢，模型数据太庞大了。
t1 = time.time()
# 装入分词模型
print("Loading models...")
tokenizer = hanlp.load('LARGE_ALBERT_BASE')
# 装入词性标注模型
# tagger = hanlp.load(hanlp.pretrained.pos.CTB5_POS_RNN_FASTTEXT_ZH)  # HanLP2.0 -->2.1
tagger = hanlp.load(hanlp.pretrained.pos.CTB9_POS_ALBERT_BASE)
# 装入依存句法分析模型
syntactic_parser = hanlp.load(hanlp.pretrained.dep.CTB7_BIAFFINE_DEP_ZH)
# 自定义词典处理、分词、词性标注、合并、依存句法分析组成 pipline， 一次过执行
parser = hanlp.pipeline() \
    .append(split_sents, output_key=('parts', 'offsets', 'words'), trie=trie) \
    .append(tokenizer, input_key='parts', output_key='tokens') \
    .append(tagger,input_key='tokens', output_key='postags') \
    .append(merge_parts, input_key=('tokens', 'postags', 'offsets', 'words'), output_key='merged') \
    .append(syntactic_parser, input_key='merged', output_key='syntactic_dependencies')
t2 = time.time()
print("Models loaded,",int(t2-t1),"s.")


# 测试文本
# text = "汤臣倍健番茄红素维生素E软胶囊"
# text = "胶瓶美汁源果粒橙12*1.25L"
text = "线路板"

# 分词
res1 = tokenizer(text)
print(res1)
# 词性标注（未引用用户自定义词典）
res2 = tagger(res1)
print(res2)
# 依存句法分析，未引用自定义词典
res_wps = []
for i in range(len(res1)):
    res_wps.append(tuple([res1[i],res2[i]]))
res3 = syntactic_parser(res_wps)
print(res3)
# 依存句法分析，引用自定义词典
res = parser(text)
print(res)
# 输出依存句法分析结果成CoNLL格式文本文件，南京大学的dependency viewer可视化展示需要utf-8编码
# f = open("D:/temp/增值税自开票2017/数据/conll-utf8.txt", "a+", encoding="UTF-8") 
# print(res.syntactic_dependencies, file = f)
# print("\n",file = f)
# f.close()




# 自定义函数， 返回依存句法分析json结果字符串
def syntacticParse(parser, text):
    res = parser(text)
    # items = res.syntactic_dependencies # HanLP 2.0 -->2.1
    items = res["syntactic_dependencies"]
    items2 = {}; items2.setdefault("items",[])
    for item in items:
        item2 = {key: value for key, value in item.items() \
                       # if key in ["cpos","deprel","form","head","id"]}    # HanLP 2.0 -->2.1             
                       if key in ["upos","deprel","form","head","id"]}
        items2.get("items").append(item2)        
    reStr = json.dumps(items2,ensure_ascii=False)
    return reStr

# 测试依存句法分析函数
reStr = syntacticParse(parser, text)


# 建立语法树（图）函数
def getTree(i,text):
    res = json.loads(text)
    try:
        items = res["items"]
    except KeyError:
        print(i,text)
        return None
    ids = []; pos = []; words = []; head = []; deprel = []
    for item in items:
        # ids.append(item["id"]); words.append(item["form"]); pos.append(item["cpos"])    # HanLP 2.0 -->2.1     
        ids.append(item["id"]); words.append(item["form"]); pos.append(item["upos"])
        head.append(item["head"]); deprel.append(item["deprel"])
    vs = pd.DataFrame({"id":ids,"word":words,"postag":pos})
    es = pd.DataFrame({"id":ids,"head":head,"deprel":deprel})
    vd = vs.to_dict(orient='records')
    ed = es.to_dict(orient='records')
    g = Graph.DictList(vd,ed, vertex_name_attr="id",edge_foreign_keys=('id', 'head'),directed=True)
    # 给根节点赋非空值
    vs = g.vs.select(id=0)   
    vs["word"] = ""; vs["postag"] = ""
    g.vs["label"]=[str(v.index)+":"+v["word"]+"\n"+v["postag"] for v in g.vs]
    g.es["label"]=g.es["deprel"]
    return g


text = "汤臣倍健维生素A维生素D软胶囊-迪士尼漫威装"

# 测试依存句法分析函数
reStr = syntacticParse(parser, text)
print(text)
# 画语法树
g = getTree(0,reStr)
random.seed(42)
plot(g,bbox=(400,300))


# 从根结点开始遍历语法树，取最先出现的名词作货劳名称，所以层级高的优先，同层级的在后面的优先
def get_hlmc(g, verbose = False):
    # 建立邻接表
    adj = [[n.index for n in v.neighbors("in")] for v in g.vs]
    spcialWords = re.compile("[\.|\-|%]") #特殊名称， "."软件版本号， "-"具体型号，跳过
    spcialPuncs = ["-", "/", "*", "+"]       #特殊标点符号，标示里面包含并列关系,
    skipWords = re.compile("[顶|装]")
    # 广度优先搜索遍历语法树
    def traverse(adj, paths, depth):
        # 记录遍历的深度
        depth += 1;  nxt_paths = []
        # 遍历所有的路径
        for path in paths['paths']:
            v = g.vs[path[-1]]
            # 如果不是并列词，则反转邻接表，改成同层级的后面的优先，排在后面的名词优先，
            # 发票语法结构的特点，前面的名词多数是修饰语         
            if v["word"] not in spcialPuncs and depth>1:
                adj[path[-1]].reverse()
            for idx in adj[path[-1]]:
                v1 = g.vs[idx]
                # 如 汤臣倍健多种维生素咀嚼片-迪士尼漫威装, 根结点是 漫威装
                if v1["word"] in spcialPuncs:
                    adj[path[-1]].reverse()  # 子结点中有并列关系，邻接表恢复为正常顺序。                    
                    break
            # 遍历路径中最后一个结点的所有邻居
            for nxt in adj[path[-1]]:
                v = g.vs[nxt]
                if verbose:
                    print(depth,v["word"],v["postag"])
                if "N"  in v["postag"]:   # 如果是名词，则加入候选货劳清单中
                    flag = True
                    if spcialWords.findall(v["word"]) and v["postag"]!= "NM" :  # 软件版本号，具体型号，跳过，药名除外。
                        flag = False
                    if v["postag"]== "NT":  # 品牌，跳过
                        flag = False
                    matched = skipWords.search(v["word"])   # 检查是否包含要跳过的特殊名词
                    if matched:
                        if matched.span()[0]+1 == len(v["word"]):   # 漫威装 在最后，跳过；安装费 在中间，接受
                            flag = False
                    if flag:
                        paths["hlmc"].append(v["word"])
                        paths["pos"].append(v["postag"])
                nxt_path = path + [nxt]   # 在路径中加入下一个结点，生成新的路径
                nxt_paths.append(nxt_path)
        # 路径的深度增加了一级
        paths['paths'] = nxt_paths                            
        if len(nxt_paths)==0:
            # 没有新路径了
            return paths
        else:
            # 往路径深度递归调用一级
            return traverse(adj, paths, depth)
    # 从根结点开始递归遍历语法树
    v = g.vs.select(id=0)[0]
    # 返回候选货劳名称列表，排在前面的在语法树中的层级较高，同层级中则排在较后的优先级较高
    res = traverse(adj, {'paths': [[v.index]], 'hlmc': [], "pos": []}, 0)
    # 没有找到名词，返回一级结点作货劳名称
    if len(res["hlmc"]) == 0:
        v1 = v.neighbors("in")[0]
        res["hlmc"].append(v1["word"]); res["pos"].append(v1["postag"])
    
    return res
    

#  测试遍历语法树得到货劳名称
hlmc = get_hlmc(g, True)
print(hlmc["hlmc"][0])


# 语义依存分析
semantic_parser = hanlp.load(hanlp.pretrained.sdp.SEMEVAL16_NEWS_BIAFFINE_ZH)
parser2 = hanlp.pipeline() \
    .append(split_sents, output_key=('parts', 'offsets', 'words'), trie=trie) \
    .append(tokenizer, input_key='parts', output_key='tokens') \
    .append(tagger,input_key='tokens', output_key='postags') \
    .append(merge_parts, input_key=('tokens', 'postags', 'offsets', 'words'), output_key='merged') \
    .append(semantic_parser, input_key='merged', output_key='semantic_dependencies')
    

# 自定义函数， 返回语义依存分析json结果字符串
def semanticParse(parser, text):
    res = parser2(text)
    # items = res.semantic_dependencies    # HanLP2.0 -->2.1   
    items = res["semantic_dependencies"]
    items2 = {}; items2.setdefault("items",[])
    for item in items:
        item2 = {key: value for key, value in item.items() \
                       # if key in ["cpos","deprel","form","head","id"]}  # HanLP2.0 -->2.1               
                       if key in ["cpos","deprel","form","head","id"]}
        items2.get("items").append(item2)       
    reStr = json.dumps(items2,ensure_ascii=False)
    return reStr



# 建立语义网络（图）函数， 语义分析结果为一个有向无环图，称为语义依存图（Semantic Dependency Graph）。
def getTree2(i,text):
    res = json.loads(text)
    try:
        items = res["items"]
    except KeyError:
        print(i,text)
        return None
    ids = []; pos = []; words = []; head = []; deprel = []
    for item in items:
        # ids.append(item["id"]); words.append(item["form"]); pos.append(item["cpos"])  # HanLP2.0 -->2.1      
        ids.append(item["id"]); words.append(item["form"]); pos.append(item["cpos"])
        head.append(item["head"]); deprel.append(item["deprel"])
    vs = pd.DataFrame({"id":ids,"word":words,"postag":pos})
    # 语义依存存在一对多的情况，每个对应关系展开成一条边。 head与deprel为list。
    ids2 = []; head2 = []; deprel2 = []
    for i in range(len(ids)):
        for j in range(len(head[i])):
            ids2.append(ids[i]); head2.append(head[i][j]); deprel2.append(deprel[i][j])
    es = pd.DataFrame({"id":ids2,"head":head2,"deprel":deprel2})
    vd = vs.to_dict(orient='records')
    ed = es.to_dict(orient='records')
    g = Graph.DictList(vd,ed, vertex_name_attr="id",edge_foreign_keys=('id', 'head'),directed=True)
    # 给根节点赋非空值
    vs = g.vs.select(id=0)   
    vs["word"] = ""; vs["postag"] = ""
    g.vs["label"]=[str(v.index)+":"+v["word"]+"\n"+v["postag"] for v in g.vs]
    g.es["label"]=g.es["deprel"]
    return g


# 测试语义依存分析函数
# text = "蜡烛两头烧"
text = "利尔康牌95%酒精消毒液"
text = "汤臣倍健番茄红素维生素E软胶囊"
text = "胶瓶美汁源果粒橙12*1.25L"
text = "细菌测定系统随机体外诊断试剂板+样本稀释液+药敏接种培养液+药敏指示剂"
# text = "蜡烛两头烧"
reStr = semanticParse(parser2, text)    
print(reStr)
# 画语义依存图， 语义分析结果为一个有向无环图，称为语义依存图（Semantic Dependency Graph）。
g = getTree2(0,reStr)
random.seed(42)
plot(g,bbox=(400,300))

#  遍历语法树得到货劳名称
print(text)
hlmc = get_hlmc(g, True)
print(hlmc["hlmc"][0])


# 命名实体识别
# recognizer = hanlp.load(hanlp.pretrained.ner.MSRA_NER_BERT_BASE_ZH)
# recognizer(list("利尔康牌95%酒精消毒液"))
# recognizer(list("汤臣倍健番茄红素维生素E软胶囊"))
# recognizer(list("联想Thinkpad X1 Carbon笔记本电脑"))



# 句法依存分析前对汇总后的货物劳务名称做预处理
def preprocess(row):     
    text = row["hwmc"]
    # 去掉前后空格等特殊处理
    text = text.strip()
    if text == "." or text == "/"  or text == "-":
        text = ""    
    # 去掉几种括号及其中内容，如果去掉后没有内容了，则予以保留
    text1 = text
    text = re.sub("\\(.*?\\)|（.*?）", "", text)
    if len(text) == 0:
        text = text1
    text1 = text
    text = re.sub("\\[.*?\\]|【.*?】", "", text)
    if len(text) == 0:
        text = text1
    text1 = text
    text = re.sub("\\{.*?\\}|｛.*?｝", "", text)
    if len(text) == 0:
        text = text1
    # 去掉商品及货物分类简称
    text1 = text        
    text = re.sub("\\*.*?\\*", "", text)
    if len(text) == 0:
        text = text1
    # 去掉单边括号后的内容，这种情况不少
    text1 = text        
    text = text.split("（")[0]  #	722 维达（Vinda 	822	荣耀8全网通（FRD 	825	彩电挂式安装调试卡（
    if len(text) == 0:
        text = text1
    text1 = text                
    text = text.split("）")[0] 
    if len(text) == 0:
        text = text1  
    text1 = text                
    text = text.split("(")[0]  
    if len(text) == 0:
        text = text1  
    text1 = text                
    text = text.split(")")[0]  
    if len(text) == 0:
        text = text1  
    text1 = text
    text = re.sub("及", " ", text)  # 国际快件费及附加 -> 国际快件费 附加
    if len(text) == 0:
        text = text1        
    text1 = text                
    text = text.split(" ")[0]  # 空格分隔的多个并列，只留第一个，如 "小米5C 移动版 3"
    if len(text) == 0:
        text = text1        
    text = re.sub("[0-9]L", "", text)  # 胶瓶美汁源果粒橙12*1.25L，去掉L，引起分词错误 
    
    return text


# 批量分析处理汇总货物劳务名称
# 重新读入货物劳务汇总
types = {"hwmc":str,"count":int,"je":float}
hlhz = pd.read_csv("D:/temp/data/增值税自开票2017/hlhz-gbk-1k.csv", sep='~',dtype=types, \
                   encoding="GBK",header=None)
hlhz.columns = ["hwmc","count","je"]
# 统计，透视表需要有 index列，增加 flag列以便作全表统计
hlhz["flag"] = 1
hltj = pd.pivot_table(hlhz, index=["flag"], values=['count',"je"],aggfunc=["sum"]).reset_index()
# 每个子集的大小及划分的子集数，先取频数最大的前1万行测试
chunk  = 1000
blocks = math.ceil(len(hlhz["hwmc"])/chunk)

# 第一个子集
hlhz1 = hlhz[:chunk]
# 统计所占的比例
hltj1 = pd.pivot_table(hlhz1, index=["flag"], values=['count',"je"],aggfunc=["sum"]).reset_index()
print(round(hltj1.iloc[0,1]/hltj.iloc[0,1]*100,1), round(hltj1.iloc[0,2]/hltj.iloc[0,2]*100,1))
# 预处理货劳名称，以便后面更准确的做依存句法分析，这里先用频数最高的前1万行测试。
hlhz1["hwmc2"] = hlhz1.apply(lambda x:preprocess(x), axis=1)

# 提取货劳名称
hlhz1 = parseHlmc(hlhz1)

# 存盘
hlhz1.to_csv("D:/temp/data/增值税自开票2017/hlhz-hanlp-utf8-1k.csv",sep='~',encoding="utf-8", \
            index=False,header=None)  


def parseHlmc(hlhz1):
    # 分别存放返回的json结果，识别的货物劳务名称与它的词性
    restr = []; reshl = []; pos = []; method = []
    Re1 = re.compile("-")
    Re2 = re.compile(r"[1-9a-zA-Z][-]")
    Re3 = re.compile(r"[-][1-9a-zA-Z]")
    Re4 = re.compile(r"[1-9a-zA-Z][-][1-9a-zA-Z]")
    t1 = time.time()
    for i in range(len(hlhz1["hwmc2"])):
        # i=43   #这一行， 在图上形成了循环，没有 id=0的根结点，引起出错，这种情况有一些。
        # "-" 并列式，先用语义分析尝试，效果要好一点
        if Re1.search(hlhz1["hwmc2"].iloc[i]) and not Re4.search(hlhz1["hwmc2"].iloc[i]):
            try: # 用语义依存分析再尝试，有向无环图。   
                text = re.sub("-", "*", hlhz1["hwmc2"].iloc[i])  # 替换"-"为"*"，以帮助正确分词
                reStr = semanticParse(parser2, text)    
                g = getTree2(i,reStr)     # 转换成语法树          
                hlmc = get_hlmc(g, False) # 遍历语法树得到货物劳务名称
                reshl.append(hlmc["hlmc"][0]); pos.append(hlmc["pos"][0]); method.append(1)
            except Exception:  #KeyError:
                try: # 调用依存句法分析再尝试。            
                    reStr = syntacticParse(parser, hlhz1["hwmc2"].iloc[i])
                    g = getTree2(i,reStr)     # 转换成语法树          
                    hlmc = get_hlmc(g, False) # 遍历语法树得到货物劳务名称
                    reshl.append(hlmc["hlmc"][0]); pos.append(hlmc["pos"][0]);  method.append(0)
                except Exception:  # 图上形成了循环，没有 id=0的根结点，引起出错。
                    reshl.append(None); pos.append(None); reStr = None ;  method.append(None)
        else:
            try:    # 调用依存句法分析
                reStr = syntacticParse(parser, hlhz1["hwmc2"].iloc[i])
                g = getTree(i,reStr)         # 转换成语法树        
                hlmc = get_hlmc(g, False)    # 遍历语法树得到货物劳务名称
                reshl.append(hlmc["hlmc"][0]); pos.append(hlmc["pos"][0]); method.append(0)
            except Exception:   #图上形成了循环，没有 id=0的根结点，引起出错。     
                try: # 用语义依存分析再尝试，有向无环图。            
                    reStr = semanticParse(parser2, hlhz1["hwmc2"].iloc[i])    
                    g = getTree2(i,reStr)     # 转换成语法树          
                    hlmc = get_hlmc(g, False) # 遍历语法树得到货物劳务名称
                    reshl.append(hlmc["hlmc"][0]); pos.append(hlmc["pos"][0]); method.append(1)
                except Exception:  #KeyError:
                    reshl.append(None); pos.append(None); reStr = None; method.append(None)
        restr.append(reStr)
        t2 = time.time()
        print(i,int(t2-t1),reshl[i],pos[i],method[i],hlhz1["hwmc"].iloc[i])

    #合并进汇总货劳名称子集
    hlhz1["hlmc"] = reshl; hlhz1["pos"] = pos; hlhz1["reStr"] = restr; hlhz1["method"] = method
    return hlhz1



# 第一个子集
hlhz1 = hlhz[:100]
# 预处理货劳名称，以便后面更准确的做依存句法分析，这里先用频数最高的前1万行测试。
# 统计所占的比例
hltj1 = pd.pivot_table(hlhz1, index=["flag"], values=['count',"je"],aggfunc=["sum"]).reset_index()
print(round(hltj1.iloc[0,1]/hltj.iloc[0,1]*100,1))
print(round(hltj1.iloc[0,2]/hltj.iloc[0,2]*100,1))

# 预处理
hlhz1["hwmc2"] = hlhz1.apply(lambda x:preprocess(x), axis=1)
# 提取货劳名称
hlhz1 = parseHlmc(hlhz1)

# 存盘
hlhz1.to_csv("D:/temp/data/增值税自开票2017/hlhz-hanlp-utf8.csv",sep='~',encoding="utf-8", \
            index=False,header=None)  

hlhz2 = hlhz1[hlhz1["hwmc2"].str.contains("/")]    
hlhz2 = hlhz1[hlhz1["hwmc2"].str.contains("及")]


# 提取错误的
hlhz3 = hlhz1.iloc[[57,174,177,182,358,402,440,458,468,473,532,557,569,653,658,673,\
                    704,788,797,822,823,828,829,894,945,976],0:]
# 可改进的
hlhz4 = hlhz1.iloc[[76,133,153,159,174,232,236,319,353,365,382,425,486,678,768,917,],0:]

hlhz5 = hlhz1[hlhz1["method"]==1]



# 重新读入货物劳务汇总依存句法分析结果
types = {"hwmc":str,"count":int,"je":float,"flag":int,"hwmc2":str,"hlmc":str,"pos":str,"reStr":str}
hlhz1 = pd.read_csv("D:/temp/增值税自开票2017/数据/hlhz-hanlp-utf8.csv", sep='~',dtype=types, \
                   encoding="utf-8",header=None)
hlhz1.columns = ["hwmc","count","je","flag","hwmc2","hlmc","pos","reStr","method"]

# 重建所有语法树，追加一列，30s
ts = []
t1 = time.time()
for i in range(len(hlhz1["hwmc"])):
    try:
        g = getTree(i,hlhz1["reStr"].iloc[i])
        ts.append(g)
    except Exception:
        try:
            g = getTree2(i,hlhz1["reStr"].iloc[i])
            ts.append(g) 
        except Exception:
            print(i,hlhz1["reStr"].iloc[i])
            ts.append(None) 
    t2 = time.time()
    print(i, int(t2-t1))
    
hlhz1["gt"] = ts      

g= hlhz1["gt"].iloc[43]
random.seed(42)
plot(g,bbox=(400,300))
print(hlhz1["reStr"].iloc[43])

dicRelation = {}
t1 = time.time()
for i in range(len(hlhz1["gt"])):
    #i = 1784
    g= hlhz1["gt"].iloc[i]
    if g is not None:
        for label in g.es["deprel"]:
            try:
                dicRelation[label] = dicRelation[label] + 1
            except Exception:
                dicRelation[label] = 1
    t2 = time.time()
    print(i,int(t2-t1))
    
wordRelations = pd.DataFrame({"relation":list(dicRelation.keys()),"count":list(dicRelation.values())})
wordRelations = wordRelations.sort_values(["count"], ascending=False)
wordRelations = wordRelations.reset_index(drop=True)  
wordRelations.to_csv("D:/temp/增值税自开票2017/数据/hlhz-hanlp-sync.csv")



# 输出依存句法分析结果成CoNLL格式文本文件，南京大学的dependency viewer可视化展示需要utf-8编码
f = open("D:/temp/增值税自开票2017/数据/hlhz-conll-utf8.txt", "a+", encoding="UTF-8") 
t1 = time.time()
for i in range(len(hlhz1["hwmc2"])):
# for i in range(10):    
    res = parser(hlhz1["hwmc2"].iloc[i])
    print(res.syntactic_dependencies, file = f)
    print("\n",file = f)
    t2 = time.time()
    print(i, int(t2-t1))
f.close()